<?php

namespace Drupal\texts;

use Drupal\Component\Gettext\PoItem;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;

class TextsTranslator implements TextsTranslatorInterface {

  /**
   * @var \Drupal\texts\TextsStorageInterface
   */
  protected $textsStorage;

  /**
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  public function __construct(EntityTypeManagerInterface $entity_type_manager, LanguageManagerInterface $language_manager) {
    $this->textsStorage = $entity_type_manager->getStorage('texts');
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function trans(string $key, array $args = [], array $options = []): string {

    $context = 'default';
    if (!empty($options['context'])) {
      $context = $options['context'];
    }

    $text = $this->textsStorage->loadByKey($key, $context);
    $current_language_code = $this->languageManager->getCurrentLanguage()->getId();
    $translation = '';

    if ($text) {
      if ($text->hasTranslation($current_language_code)) {
        $text = $text->getTranslation($current_language_code);
      }
      $translation = $text->getTranslationText();
    }

    // Fallback to provided default translation.
    if (empty($text) && !empty($options['default_translation'])) {
      $translation = $options['default_translation'];

      // Add new text to database.
      $new_text_data = [
        'key' => $key,
        'context' => $context,
        'translation' => $translation,
      ];
      if ($this->isPlural($translation)) {
        $new_text_data['plural'] = TRUE;
      }
      $new_text = $this->textsStorage->create($new_text_data);
      $new_text->save();
    }

    // Use TranslatableMarkup to replace all placeholders.
    if ($args) {
      $translatable_markup = new TextsTranslatableMarkup($translation, $args, $options);

      if ($this->isPlural($translation)) {
        $plurals = $this->getPluralsParts($translation);
        $translatable_markup = new TextsPluralTranslatableMarkup($options['count'], $plurals[0], $plurals[1], $args, $options);
      }
      $translation = $translatable_markup->__toString();
    }

    return $translation;
  }

  /**
   * {@inheritdoc}
   */
  public function formatPlural(int $count, string $key, string $singular = '', string $plural = '', array $args = [], array $options = []): string {
    $options['default_translation'] = $this->getPluralMerged(
      [$singular, $plural]
    );
    $options['count'] = $count;
    $translation = $this->trans($key, $args, $options);
    return $translation;
  }

  public function isPlural($string): bool {
    $is_plural = FALSE;
    if (strpos($string, PoItem::DELIMITER) !== FALSE) {
      $is_plural = TRUE;
    }
    return $is_plural;
  }

  /**
   * Check if the string is a plural string.
   *
   * @param $string
   *
   * @return array|string[]
   */
  public function getPluralsParts($string) {
    $result = [$string, $string];
    if ($this->isPlural($string)) {
      $result = explode(PoItem::DELIMITER, $string);
    }
    return $result;
  }

  /**
   * Merge plural string parts.
   *
   * @param $elements
   *
   * @return string
   */
  public function getPluralMerged($elements) {
    return implode(PoItem::DELIMITER, $elements);
  }

}
